@@ -52,6 +52,7 @@ package com.openexchange.client.onboarding.mail;
 import java.util.Collections;
 import java.util.EnumSet;
 import java.util.HashMap;
+import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.logging.Logger;
@@ -72,7 +73,9 @@ import com.openexchange.client.onboarding.plist.PlistResult;
 import com.openexchange.exception.OXException;
 import com.openexchange.groupware.ldap.User;
 import com.openexchange.groupware.userconfiguration.Permission;
+import com.openexchange.i18n.tools.StringHelper;
 import com.openexchange.java.Strings;
+import com.openexchange.mail.MailExceptionCode;
 import com.openexchange.mail.api.MailConfig;
 import com.openexchange.mail.service.MailService;
 import com.openexchange.mail.transport.config.TransportAuthSupportAware;
@@ -81,12 +84,14 @@ import com.openexchange.mail.usersetting.UserSettingMail;
 import com.openexchange.mail.usersetting.UserSettingMailStorage;
 import com.openexchange.mailaccount.MailAccount;
 import com.openexchange.mailaccount.MailAccountStorageService;
+import com.openexchange.osgi.ServiceListing;
 import com.openexchange.plist.PListDict;
 import com.openexchange.server.ServiceExceptionCode;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.session.Session;
 import com.openexchange.sessiond.SessionMatcher;
 import com.openexchange.sessiond.SessiondService;
+import com.openexchange.tools.session.ServerSession;
 import com.openexchange.user.UserService;
 
 /**
@@ -98,24 +103,31 @@ import com.openexchange.user.UserService;
  */
 public class MailOnboardingProvider implements OnboardingPlistProvider {
 
+    private static final Logger LOG = Logger.getLogger(MailOnboardingProvider.class.getName());
+
+    private final ServiceListing<CustomLoginSource> customLoginSources;
     private final ServiceLookup services;
     private final String identifier;
     private final Set<Device> supportedDevices;
     private final Set<OnboardingType> supportedTypes;
 
-    private static final Logger LOG = Logger.getLogger(MailOnboardingProvider.class.getName());
-
     /**
      * Initializes a new {@link MailOnboardingProvider}.
      */
-    public MailOnboardingProvider(ServiceLookup services) {
+    public MailOnboardingProvider(ServiceListing<CustomLoginSource> customLoginSources, ServiceLookup services) {
         super();
+        this.customLoginSources = customLoginSources;
         this.services = services;
         identifier = BuiltInProvider.MAIL.getId();
         supportedDevices = EnumSet.allOf(Device.class);
         supportedTypes = EnumSet.of(OnboardingType.PLIST, OnboardingType.MANUAL);
     }
 
+    private CustomLoginSource getHighestRankedCustomLoginSource() {
+        List<CustomLoginSource> loginSources = customLoginSources.getServiceList();
+        return loginSources.isEmpty() ? null : loginSources.get(0);
+    }
+
     @Override
     public String getDescription() {
         return "Configures IMAP/SMTP.";
@@ -179,7 +191,7 @@ public class MailOnboardingProvider implements OnboardingPlistProvider {
         }
 
         Scenario scenario = request.getScenario();
-        if (!Device.getActionsFor(device, scenario.getType(), session).contains(request.getAction())) {
+        if (!Device.getActionsFor(request.getClientDevice(), device, scenario.getType(), session).contains(request.getAction())) {
             throw OnboardingExceptionCodes.UNSUPPORTED_ACTION.create(request.getAction().getId());
         }
 
@@ -195,40 +207,64 @@ public class MailOnboardingProvider implements OnboardingPlistProvider {
         }
     }
 
-    private Configurations getEffectiveConfigurations(Session session) throws OXException {
+    private Configurations getEffectiveConfigurations(MailConfig optImapConfig, Session session) throws OXException {
         // IMAP
         Configuration imapConfiguration;
         {
-            MailConfig imapConfig = services.getService(MailService.class).getMailConfig(session, MailAccount.DEFAULT_ID);
-            String imapServer = OnboardingUtility.getValueFromProperty("com.openexchange.client.onboarding.mail.imap.host", null, session);
-            if (null == imapServer) {
-                imapServer = imapConfig.getServer();
-            }
-            Integer imapPort = OnboardingUtility.getIntFromProperty("com.openexchange.client.onboarding.mail.imap.port", null, session);
-            if (null == imapPort) {
-                imapPort = Integer.valueOf(imapConfig.getPort());
-            }
-            Boolean imapSecure = OnboardingUtility.getBoolFromProperty("com.openexchange.client.onboarding.mail.imap.secure", null, session);
-            if (null == imapSecure) {
-                imapSecure = Boolean.valueOf(imapConfig.isSecure());
-            }
+            MailConfig imapConfig = null == optImapConfig ? services.getService(MailService.class).getMailConfig(session, MailAccount.DEFAULT_ID) : optImapConfig;
+            Configuration.Builder configBuilder = Configuration.builder();
 
-            Boolean customSource = OnboardingUtility.getBoolFromProperty("com.openexchange.client.onboarding.mail.imap.login.customsource", false, session);
-            String imapLogin;
-
-            if (customSource) {
-                CustomLoginSource customLoginSource = services.getService(CustomLoginSource.class);
-                if(customLoginSource==null){
-                    LOG.warning("Unable to find any CustomLoginSource services! Falling back to imap config.");
-                    imapLogin = imapConfig.getLogin();
-                } else {
-                    imapLogin = customLoginSource.getImapLogin(session);
+            // Host
+            {
+                String imapServer = OnboardingUtility.getValueFromProperty("com.openexchange.client.onboarding.mail.imap.host", null, session);
+                if (null == imapServer) {
+                    imapServer = imapConfig.getServer();
                 }
-            } else {
-                imapLogin = imapConfig.getLogin();
+                configBuilder.withHost(imapServer);
             }
-            String imapPassword = imapConfig.getPassword();
-            imapConfiguration = new Configuration(imapServer, imapPort.intValue(), imapSecure.booleanValue(), imapLogin, imapPassword);
+
+            // Port
+            {
+                Integer imapPort = OnboardingUtility.getIntFromProperty("com.openexchange.client.onboarding.mail.imap.port", null, session);
+                if (null == imapPort) {
+                    imapPort = Integer.valueOf(imapConfig.getPort());
+                }
+                configBuilder.withPort(imapPort.intValue());
+            }
+
+            // Secure (SSL/TLS)
+            {
+                Boolean imapSecure = OnboardingUtility.getBoolFromProperty("com.openexchange.client.onboarding.mail.imap.secure", null, session);
+                if (null == imapSecure) {
+                    imapSecure = Boolean.valueOf(imapConfig.isSecure());
+                }
+                configBuilder.withSecure(imapSecure.booleanValue());
+            }
+
+            // Login
+            {
+                String imapLogin;
+                Boolean customSource = OnboardingUtility.getBoolFromProperty("com.openexchange.client.onboarding.mail.imap.login.customsource", Boolean.FALSE, session);
+                if (customSource.booleanValue()) {
+                    CustomLoginSource customLoginSource = getHighestRankedCustomLoginSource();
+                    if (customLoginSource == null) {
+                        LOG.warning("Unable to find any CustomLoginSource services! Falling back to imap config.");
+                        imapLogin = imapConfig.getLogin();
+                    } else {
+                        imapLogin = customLoginSource.getImapLogin(session);
+                    }
+                } else {
+                    imapLogin = imapConfig.getLogin();
+                }
+                configBuilder.withLogin(imapLogin);
+            }
+
+            // Password
+            {
+                String imapPassword = imapConfig.getPassword();
+                configBuilder.withPassword(imapPassword);
+            }
+            imapConfiguration = configBuilder.build();
         }
 
         // SMTP
@@ -236,40 +272,67 @@ public class MailOnboardingProvider implements OnboardingPlistProvider {
         {
             TransportConfig smtpConfig = services.getService(MailService.class).getTransportConfig(session, MailAccount.DEFAULT_ID);
             TransportConfig.getTransportConfig(smtpConfig, session, MailAccount.DEFAULT_ID);
-            String smtpServer = OnboardingUtility.getValueFromProperty("com.openexchange.client.onboarding.mail.smtp.host", null, session);
-            if (null == smtpServer) {
-                smtpServer = smtpConfig.getServer();
-            }
-            Integer smtpPort = OnboardingUtility.getIntFromProperty("com.openexchange.client.onboarding.mail.smtp.port", null, session);
-            if (null == smtpPort) {
-                smtpPort = Integer.valueOf(smtpConfig.getPort());
-            }
-            Boolean smtpSecure = OnboardingUtility.getBoolFromProperty("com.openexchange.client.onboarding.mail.smtp.secure", null, session);
-            if (null == smtpSecure) {
-                smtpSecure = Boolean.valueOf(smtpConfig.isSecure());
-            }
-            String smtpPassword = smtpConfig.getPassword();
+            Configuration.Builder configBuilder = Configuration.builder();
 
-            Boolean customSource = OnboardingUtility.getBoolFromProperty("com.openexchange.client.onboarding.mail.smtp.login.customsource", false, session);
-            String smtpLogin;
-
-            if (customSource) {
-                CustomLoginSource customLoginSource = services.getService(CustomLoginSource.class);
-                if(customLoginSource==null){
-                    LOG.warning("Unable to find any CustomLoginSource services! Falling back to smtp config.");
-                    smtpLogin = smtpConfig.getLogin();
-                }else {
-                    smtpLogin = customLoginSource.getSmtpLogin(session);
+            // Host
+            {
+                String smtpServer = OnboardingUtility.getValueFromProperty("com.openexchange.client.onboarding.mail.smtp.host", null, session);
+                if (null == smtpServer) {
+                    smtpServer = smtpConfig.getServer();
                 }
-            } else {
-                smtpLogin = smtpConfig.getLogin();
+                configBuilder.withHost(smtpServer);
             }
 
-            if ((smtpConfig instanceof TransportAuthSupportAware) && (false == ((TransportAuthSupportAware) smtpConfig).isAuthSupported())) {
-                smtpLogin = null;
-                smtpPassword = null;
+            // Port
+            {
+                Integer smtpPort = OnboardingUtility.getIntFromProperty("com.openexchange.client.onboarding.mail.smtp.port", null, session);
+                if (null == smtpPort) {
+                    smtpPort = Integer.valueOf(smtpConfig.getPort());
+                }
+                configBuilder.withPort(smtpPort.intValue());
             }
-            smtpConfiguration = new Configuration(smtpServer, smtpPort.intValue(), smtpSecure.booleanValue(), smtpLogin, smtpPassword);
+
+            // Secure (SSL/TLS)
+            {
+                Boolean smtpSecure = OnboardingUtility.getBoolFromProperty("com.openexchange.client.onboarding.mail.smtp.secure", null, session);
+                if (null == smtpSecure) {
+                    smtpSecure = Boolean.valueOf(smtpConfig.isSecure());
+                }
+                configBuilder.withSecure(smtpSecure.booleanValue());
+            }
+
+            // Password
+            {
+                String smtpPassword = smtpConfig.getPassword();
+                configBuilder.withPassword(smtpPassword);
+            }
+
+            // Login
+            {
+                String smtpLogin;
+                Boolean customSource = OnboardingUtility.getBoolFromProperty("com.openexchange.client.onboarding.mail.smtp.login.customsource", Boolean.FALSE, session);
+                if (customSource.booleanValue()) {
+                    CustomLoginSource customLoginSource = getHighestRankedCustomLoginSource();
+                    if (customLoginSource == null) {
+                        LOG.warning("Unable to find any CustomLoginSource services! Falling back to smtp config.");
+                        smtpLogin = smtpConfig.getLogin();
+                    } else {
+                        smtpLogin = customLoginSource.getSmtpLogin(session);
+                    }
+                } else {
+                    smtpLogin = smtpConfig.getLogin();
+                }
+                configBuilder.withLogin(smtpLogin);
+            }
+
+            // Authentication needed?
+            {
+                if ((smtpConfig instanceof TransportAuthSupportAware) && (false == ((TransportAuthSupportAware) smtpConfig).isAuthSupported())) {
+                    configBuilder.withNeedsAuthentication(false);
+                }
+            }
+
+            smtpConfiguration = configBuilder.build();
         }
 
         // Return configurations
@@ -287,30 +350,36 @@ public class MailOnboardingProvider implements OnboardingPlistProvider {
             throw OXException.general("No active session for user " + userId + " in context " + contextId);
         }
 
-        if (session.getPassword() == null) {
-            // Try to find a session with password!=null
+        // Check if session is suitable to obtain the MailConfig instance
+        MailConfig mailConfig = null;
+        try {
+            mailConfig = services.getService(MailService.class).getMailConfig(session, MailAccount.DEFAULT_ID);
+        } catch (OXException e) {
+            if (!MailExceptionCode.MISSING_CONNECT_PARAM.equals(e) || session.getPassword() != null) {
+                throw e;
+            }
+
+            // Need to find a session with a password; otherwise no MailConfig can be obtained
             SessionMatcher matcher = new SessionMatcher() {
 
                 @Override
                 public Set<Flag> flags() {
-                    return SessionMatcher.NO_FLAGS;
+                    return SessionMatcher.ALL_FLAGS;
                 }
 
                 @Override
                 public boolean accepts(Session session) {
-                    if (session.getPassword() != null) {
-                        return true;
-                    }
-                    return false;
+                    return session.getPassword() != null;
                 }
             };
-            session = sessiondService.findFirstMatchingSessionForUser(userId, contextId, matcher);
-            if (null == session) {
-                throw OXException.general("No active session for user " + userId + " in context " + contextId);
+            Session sessionWithPassword = sessiondService.findFirstMatchingSessionForUser(userId, contextId, matcher);
+            if (null == sessionWithPassword) {
+                throw OXException.general("No suitable session found for user " + userId + " in context " + contextId);
             }
+            session = sessionWithPassword;
         }
 
-        return getEffectiveConfigurations(session);
+        return getEffectiveConfigurations(mailConfig, session);
     }
 
     private Result doExecutePlist(OnboardingRequest request, Result previousResult, Session session) throws OXException {
@@ -333,7 +402,7 @@ public class MailOnboardingProvider implements OnboardingPlistProvider {
     private final static String SMTP_SECURE_FIELD = "smtpSecure";
 
     private Result displayResult(OnboardingRequest request, Result previousResult, Session session) throws OXException {
-        Configurations configurations = getEffectiveConfigurations(session);
+        Configurations configurations = getEffectiveConfigurations(null, session);
 
         Map<String, Object> configuration = null == previousResult ? new HashMap<String, Object>(8) : ((DisplayResult) previousResult).getConfiguration();
 
@@ -342,14 +411,23 @@ public class MailOnboardingProvider implements OnboardingPlistProvider {
         configuration.put(IMAP_PORT_FIELD, new Integer(configurations.imapConfig.port));
         configuration.put(IMAP_SECURE_FIELD, new Boolean(configurations.imapConfig.secure));
 
-        configuration.put(SMTP_LOGIN_FIELD, configurations.smtpConfig.login);
-        configuration.put(SMTP_SERVER_FIELD, configurations.smtpConfig.host);
+        boolean needsAuthentication = false == configurations.smtpConfig.noAuthentication();
+        if (needsAuthentication) {
+            configuration.put(SMTP_LOGIN_FIELD, configurations.smtpConfig.login);
+            configuration.put(SMTP_SERVER_FIELD, configurations.smtpConfig.host);
+        } else {
+            String none = StringHelper.valueOf(getUser(session).getLocale()).getString("None");
+            configuration.put(SMTP_LOGIN_FIELD, none);
+            configuration.put(SMTP_SERVER_FIELD, none);
+        }
         configuration.put(SMTP_PORT_FIELD, new Integer(configurations.smtpConfig.port));
         configuration.put(SMTP_SECURE_FIELD, new Boolean(configurations.smtpConfig.secure));
 
         return new DisplayResult(configuration, ResultReply.NEUTRAL);
     }
 
+
+
     // --------------------------------------------- PLIST utils --------------------------------------------------------------
 
     private UserSettingMail getUserSettingMail(int userId, int contextId) throws OXException {
@@ -357,6 +435,14 @@ public class MailOnboardingProvider implements OnboardingPlistProvider {
         return UserSettingMailStorage.getInstance().getUserSettingMail(userId, contextId);
     }
 
+    private User getUser(Session session) throws OXException {
+        if (session instanceof ServerSession) {
+            return ((ServerSession) session).getUser();
+        }
+
+        return getUser(session.getUserId(), session.getContextId());
+    }
+
     private User getUser(int userId, int contextId) throws OXException {
         UserService service = services.getService(UserService.class);
         if (service == null) {
@@ -407,6 +493,7 @@ public class MailOnboardingProvider implements OnboardingPlistProvider {
         // Designates the full email address for the account. If not present in the payload, the device prompts for this string during profile installation.
         payloadContent.addStringValue("EmailAddress", getUserSettingMail(userId, contextId).getSendAddr());
 
+        // -------------------------------------------------- IncomingMailServer --------------------------------------------------------
 
         // Designates the authentication scheme for incoming mail. Allowed values are EmailAuthPassword and EmailAuthNone.
         payloadContent.addStringValue("IncomingMailServerAuthentication", "EmailAuthPassword");
@@ -424,8 +511,11 @@ public class MailOnboardingProvider implements OnboardingPlistProvider {
         // If not present in the payload, and the account is set up to require authentication for incoming email, the device will prompt for this string during profile installation.
         payloadContent.addStringValue("IncomingMailServerUsername", configurations.imapConfig.login);
 
+        // -------------------------------------------------- OutgoingMailServer --------------------------------------------------------
+
         // Designates the authentication scheme for outgoing mail. Allowed values are EmailAuthPassword and EmailAuthNone.
-        payloadContent.addStringValue("OutgoingMailServerAuthentication", (Strings.isEmpty(configurations.smtpConfig.login) && Strings.isEmpty(configurations.smtpConfig.password)) ? "EmailAuthNone" : "EmailAuthPassword");
+        boolean needsAuthentication = !Strings.isEmpty(configurations.smtpConfig.login) || !Strings.isEmpty(configurations.smtpConfig.password);
+        payloadContent.addStringValue("OutgoingMailServerAuthentication", needsAuthentication ? "EmailAuthPassword" : "EmailAuthNone");
 
         // Designates the outgoing mail server host name (or IP address).
         payloadContent.addStringValue("OutgoingMailServerHostName", configurations.smtpConfig.host);
@@ -438,7 +528,15 @@ public class MailOnboardingProvider implements OnboardingPlistProvider {
 
         // Designates the user name for the email account, usually the same as the email address up to the @ character.
         // If not present in the payload, and the account is set up to require authentication for outgoing email, the device prompts for this string during profile installation.
-        payloadContent.addStringValue("OutgoingMailServerUsername", configurations.smtpConfig.login);
+        if (needsAuthentication) {
+            payloadContent.addStringValue("OutgoingMailServerUsername", configurations.smtpConfig.login);
+
+            if (Strings.isNotEmpty(configurations.smtpConfig.password) && Strings.isNotEmpty(configurations.imapConfig.password) && configurations.smtpConfig.password.equals(configurations.imapConfig.password)) {
+                payloadContent.addBooleanValue("OutgoingPasswordSameAsIncomingPassword", true);
+            }
+        } else {
+            payloadContent.addBooleanValue("OutgoingPasswordSameAsIncomingPassword", true);
+        }
 
         // Further options (currently not used)
 
