@@ -67,6 +67,7 @@ import com.openexchange.capabilities.CapabilityService;
 import com.openexchange.capabilities.CapabilitySet;
 import com.openexchange.client.onboarding.AvailabilityResult;
 import com.openexchange.client.onboarding.BuiltInProvider;
+import com.openexchange.client.onboarding.ClientDevice;
 import com.openexchange.client.onboarding.CompositeId;
 import com.openexchange.client.onboarding.DefaultScenario;
 import com.openexchange.client.onboarding.Device;
@@ -83,9 +84,13 @@ import com.openexchange.client.onboarding.Scenario;
 import com.openexchange.client.onboarding.service.OnboardingService;
 import com.openexchange.client.onboarding.service.OnboardingView;
 import com.openexchange.exception.OXException;
+import com.openexchange.image.ImageLocation;
 import com.openexchange.java.Strings;
+import com.openexchange.log.LogProperties;
+import com.openexchange.mail.text.HtmlProcessing;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.session.Session;
+import com.openexchange.sessiond.SessiondService;
 
 /**
  * {@link OnboardingServiceImpl}
@@ -179,6 +184,11 @@ public class OnboardingServiceImpl implements OnboardingService {
 
         // Invocation chain - looping through providers
         Scenario scenario = request.getScenario();
+        if (false == request.getClientDevice().implies(request.getDevice())) {
+            throw OnboardingExceptionCodes.NO_SUCH_SCENARIO.create(scenario.getId());
+        }
+
+
         Result result = null;
         for (OnboardingProvider provider : scenario.getProviders(session)) {
             Result currentResult = provider.execute(request, result, session);
@@ -248,11 +258,15 @@ public class OnboardingServiceImpl implements OnboardingService {
     }
 
     @Override
-    public DeviceAwareScenario getScenario(String scenarioId, Device device, Session session) throws OXException {
+    public DeviceAwareScenario getScenario(String scenarioId, ClientDevice clientDevice, Device device, Session session) throws OXException {
         if (null == scenarioId) {
             throw OnboardingExceptionCodes.NO_SUCH_SCENARIO.create("null");
         }
 
+        if (false == clientDevice.implies(device)) {
+            throw OnboardingExceptionCodes.NO_SUCH_SCENARIO.create(scenarioId);
+        }
+
         Map<String, ConfiguredScenario> configuredScenarios = configuredScenariosReference.get();
         Scenario scenario = getScenario(scenarioId, configuredScenarios, session);
         if (null == scenario || !scenario.isEnabled(session)) {
@@ -269,15 +283,19 @@ public class OnboardingServiceImpl implements OnboardingService {
             missingCapabilities.addAll(result.getMissingCapabilities());
         }
 
-        return new DeviceAwareScenarionImpl(scenario, enabled, missingCapabilities, device, Device.getActionsFor(device, scenario.getType(), session));
+        return new DeviceAwareScenarionImpl(scenario, enabled, missingCapabilities, device, Device.getActionsFor(clientDevice, device, scenario.getType(), session));
     }
 
     @Override
-    public DeviceAwareScenario getScenario(String scenarioId, Device device, int userId, int contextId) throws OXException {
+    public DeviceAwareScenario getScenario(String scenarioId, ClientDevice clientDevice, Device device, int userId, int contextId) throws OXException {
         if (null == scenarioId) {
             throw OnboardingExceptionCodes.NO_SUCH_SCENARIO.create("null");
         }
 
+        if (false == clientDevice.implies(device)) {
+            throw OnboardingExceptionCodes.NO_SUCH_SCENARIO.create(scenarioId);
+        }
+
         Map<String, ConfiguredScenario> configuredScenarios = configuredScenariosReference.get();
         Scenario scenario = getScenario(scenarioId, configuredScenarios, userId, contextId);
         if (null == scenario || !scenario.isEnabled(userId, contextId)) {
@@ -292,11 +310,15 @@ public class OnboardingServiceImpl implements OnboardingService {
             enabled &= result.isAvailable();
             missingCapabilities.addAll(result.getMissingCapabilities());
         }
-        return new DeviceAwareScenarionImpl(scenario, enabled, missingCapabilities, device, Device.getActionsFor(device, scenario.getType(), userId, contextId));
+        return new DeviceAwareScenarionImpl(scenario, enabled, missingCapabilities, device, Device.getActionsFor(clientDevice, device, scenario.getType(), userId, contextId));
     }
 
     @Override
-    public List<DeviceAwareScenario> getScenariosFor(Device device, Session session) throws OXException {
+    public List<DeviceAwareScenario> getScenariosFor(ClientDevice clientDevice, Device device, Session session) throws OXException {
+        if (false == clientDevice.implies(device)) {
+            return Collections.emptyList();
+        }
+
         List<String> availableScenarios = device.getScenarios(session);
         if (null == availableScenarios || availableScenarios.isEmpty()) {
             return Collections.emptyList();
@@ -322,7 +344,7 @@ public class OnboardingServiceImpl implements OnboardingService {
                         enabled &= result.isAvailable();
                         missingCapabilities.addAll(result.getMissingCapabilities());
                     }
-                    scenarios.add(new DeviceAwareScenarionImpl(scenario, enabled, missingCapabilities, device, Device.getActionsFor(device, scenario.getType(), session)));
+                    scenarios.add(new DeviceAwareScenarionImpl(scenario, enabled, missingCapabilities, device, Device.getActionsFor(clientDevice, device, scenario.getType(), session)));
                 }
             }
         }
@@ -512,8 +534,19 @@ public class OnboardingServiceImpl implements OnboardingService {
             return null;
         }
 
+        String linkImageUrl = null;
+        {
+            ConfiguredLinkImage image = configuredLink.getImage();
+            if (null != image) {
+                // Build image location
+                ImageLocation imageLocation = new ImageLocation.Builder(image.getName()).id(image.getType().getScheme()).optImageHost(HtmlProcessing.imageHost()).build();
+                OnboardingImageDataSource imgSource = OnboardingImageDataSource.getInstance();
+                linkImageUrl = imgSource.generateUrl(imageLocation, session);
+            }
+        }
+
         if (false == configuredLink.isProperty()) {
-            return new Link(configuredLink.getUrl(), configuredLink.getType());
+            return new Link(configuredLink.getUrl(), configuredLink.getType(), linkImageUrl);
         }
 
         // Look up the actual link by retrieving the denoted property
@@ -523,7 +556,7 @@ public class OnboardingServiceImpl implements OnboardingService {
             LOG.warn("No such property providing the link: {}", configuredLink.getUrl());
             return null;
         }
-        return new Link(url, configuredLink.getType());
+        return new Link(url, configuredLink.getType(), linkImageUrl);
     }
 
     private Link resolveLink(ConfiguredLink configuredLink, int userId, int contextId) throws OXException {
@@ -531,8 +564,28 @@ public class OnboardingServiceImpl implements OnboardingService {
             return null;
         }
 
+        String linkImageUrl = null;
+        {
+            ConfiguredLinkImage image = configuredLink.getImage();
+            if (null != image) {
+                // Try to build image location
+                String optSessionId = LogProperties.get(LogProperties.Name.SESSION_SESSION_ID);
+                if (null != optSessionId) {
+                    SessiondService service = services.getOptionalService(SessiondService.class);
+                    if (null != service) {
+                        Session session = service.getSession(optSessionId);
+                        if (null != session && session.getContextId() == contextId && session.getUserId() == userId) {
+                            ImageLocation imageLocation = new ImageLocation.Builder(image.getName()).id(image.getType().getScheme()).optImageHost(HtmlProcessing.imageHost()).build();
+                            OnboardingImageDataSource imgSource = OnboardingImageDataSource.getInstance();
+                            linkImageUrl = imgSource.generateUrl(imageLocation, session);
+                        }
+                    }
+                }
+            }
+        }
+
         if (false == configuredLink.isProperty()) {
-            return new Link(configuredLink.getUrl(), configuredLink.getType());
+            return new Link(configuredLink.getUrl(), configuredLink.getType(), linkImageUrl);
         }
 
         // Look up the actual link by retrieving the denoted property
@@ -542,32 +595,34 @@ public class OnboardingServiceImpl implements OnboardingService {
             LOG.warn("No such property providing the link: {}", configuredLink.getUrl());
             return null;
         }
-        return new Link(url, configuredLink.getType());
+        return new Link(url, configuredLink.getType(), linkImageUrl);
     }
 
     @Override
-    public OnboardingView getViewFor(Session session) throws OXException {
+    public OnboardingView getViewFor(ClientDevice clientDevice, Session session) throws OXException {
         Map<Device, List<CompositeId>> availableDevices;
 
         {
             Map<String, ConfiguredScenario> configuredScenarios = configuredScenariosReference.get();
             availableDevices = new EnumMap<Device, List<CompositeId>>(Device.class);
             for (Device device : Device.values()) {
-                List<String> availableScenarios = device.getScenarios(session);
-                if (null != availableScenarios && !availableScenarios.isEmpty()) {
+                if (clientDevice.implies(device)) {
+                    List<String> availableScenarios = device.getScenarios(session);
+                    if (null != availableScenarios && !availableScenarios.isEmpty()) {
 
-                    List<CompositeId> compositeIds = availableDevices.get(device);
-                    if (null == compositeIds) {
-                        compositeIds = new ArrayList<CompositeId>(8);
-                        availableDevices.put(device, compositeIds);
-                    }
+                        List<CompositeId> compositeIds = availableDevices.get(device);
+                        if (null == compositeIds) {
+                            compositeIds = new ArrayList<CompositeId>(8);
+                            availableDevices.put(device, compositeIds);
+                        }
 
-                    for (String scenarioId : availableScenarios) {
-                        ConfiguredScenario configuredScenario = configuredScenarios.get(scenarioId);
-                        if (null != configuredScenario && configuredScenario.isEnabled()) {
-                            Scenario scenario = getScenario(configuredScenario, configuredScenarios, false, session);
-                            if (null != scenario) {
-                                compositeIds.add(new CompositeId(device, scenarioId));
+                        for (String scenarioId : availableScenarios) {
+                            ConfiguredScenario configuredScenario = configuredScenarios.get(scenarioId);
+                            if (null != configuredScenario && configuredScenario.isEnabled()) {
+                                Scenario scenario = getScenario(configuredScenario, configuredScenarios, false, session);
+                                if (null != scenario) {
+                                    compositeIds.add(new CompositeId(device, scenarioId));
+                                }
                             }
                         }
                     }
@@ -575,7 +630,7 @@ public class OnboardingServiceImpl implements OnboardingService {
             }
         }
 
-        OnboardingViewImpl view = new OnboardingViewImpl();
+        OnboardingViewImpl view = new OnboardingViewImpl(clientDevice);
         view.add(availableDevices);
         return view;
     }
